package backend

import (
	"context"
	"errors"
	"kpt-tmr-group/model"
	"kpt-tmr-group/pkg/xerr"
	operationPb "kpt-tmr-group/proto/go/backend/operation"

	"gorm.io/gorm"
)

const (
	FeedCategory = "feed"
	CowCategory  = "cow"
)

func (s *StoreEntry) CategoryData(ctx context.Context, req *operationPb.ClassRequest) error {

	pastureDetail, err := s.GetGroupPastureListById(ctx, int64(req.PastureId))
	if err != nil {
		return xerr.WithStack(err)
	}
	switch req.KeyWord {
	case FeedCategory:
		history := &model.ForageCategory{}
		if err = s.DB.Model(new(model.ForageCategory)).
			Where("pasture_id = ?", req.PastureId).
			Where("is_delete = ?", operationPb.IsShow_NO).
			Where("data_source = ? ", operationPb.DataSource_FROM_PASTURE).
			Where("parent_id = ?", req.ParentId).Where("name = ?", req.Name).
			First(history).Error; err != nil {
			if !errors.Is(err, gorm.ErrRecordNotFound) {
				return xerr.WithStack(err)
			}
		}
		if history.IsShow == operationPb.IsShow_OK && history.Id > 0 {
			if err = s.DB.Model(new(model.ForageCategory)).Where("id = ?", history.Id).Updates(map[string]interface{}{
				"number":    req.Number,
				"is_show":   req.IsShow,
				"is_delete": req.IsDelete,
			}).Error; err != nil {
				return xerr.WithStack(err)
			}
		}

		newFeedData := model.NewPastureForageCategory(req, pastureDetail)
		if err = s.DB.Model(new(model.ForageCategory)).Create(newFeedData).Error; err != nil {
			return xerr.WithStack(err)
		}
	case CowCategory:
		history := &model.CattleCategory{}
		if err = s.DB.Model(new(model.CattleCategory)).
			Where("pasture_id = ?", req.PastureId).
			Where("is_delete = ?", operationPb.IsShow_NO).
			Where("data_source = ? ", operationPb.DataSource_FROM_PASTURE).
			Where("parent_id = ?", req.ParentId).Where("name = ?", req.Name).
			First(history).Error; err != nil {
			if !errors.Is(err, gorm.ErrRecordNotFound) {
				return xerr.WithStack(err)
			}
		}
		if history.IsShow == operationPb.IsShow_OK && history.Id > 0 {
			if err = s.DB.Model(new(model.CattleCategory)).Where("id = ?", history.Id).Updates(map[string]interface{}{
				"number":    req.Number,
				"is_show":   req.IsShow,
				"is_delete": req.IsDelete,
			}).Error; err != nil {
				return xerr.WithStack(err)
			}
		}
		newCattleData := model.NewPastureCattleCategory(req, pastureDetail)
		if err = s.DB.Model(new(model.CattleCategory)).Create(newCattleData).Error; err != nil {
			return xerr.WithStack(err)
		}
	}

	return nil
}
